/* 
    All events are lightweight and share one instance of this model.

    Copyright (C) 2000 Ben Martin

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

namespace __PRIVATE_FAM_NAMESPACE {

	// make this private
	// FIXME: should make FamppEvent a friend and all methods in FamppEventModel privte.
	// egcs gcc version egcs-2.91.66 19990314/Linux (egcs-1.1.2 release)
	// gives ../../include/private/FamppEventModel.hh:4: sorry, not implemented: `namespace_decl' not supported by dump_type
	// If I try to declare parent namespace class a friend.
	class FamppEventModel {

		private:

			FAMEvent theFAMEvent;
		public:
			FAMEvent* getFAMEvent() {
				return &theFAMEvent;
			}
	};

	// private NS, no user conflict.
	static FamppEventModel theFamppEventModel;

};





