/*
    This is a simple test of the singleton code to make sure that only one fam is created/destroyed

	Always one creation and one destruction.

    Copyright (C) 2000 Ben Martin

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <fam.hh>
#include <famStatic.hh>

#include <iostream>

using namespace __FAM_NAMESPACE;


int main(int argc,char** argv ) {
	
{
	Handle<Fampp> outer_f = getInstance((Fampp*)NULL);

	{
		Handle<Fampp> f = getInstance((Fampp*)NULL);
		Handle<Fampp> f2 = getInstance((Fampp*)NULL);
		cerr << "main" << endl;
	}
}	
	return 0;
	
}
